# =====================================================================
# rpciolib.awk: default AWK I/O hooks for CSA CGI programs.
#
# Copyright (c) 2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: rpciolib.awk,v 1.5 2004/08/08 21:15:37 carlo Exp $

# =====================================================================
# string _invar(string name, string value)
#
# Default input variable post-processing hook. See rcassign() for the
# meaning of the parameters. Note that cgi.awk writes a call to
# this function for every input argument, and when no more arguments
# are available it calls it with the special argument "_ENOMSG", and the
# associated value (see csalib.awk). In this way the function knows
# that it cannot expect more args, and if any required ones are still
# missing it can take proper actions, if desired. The special array
# _RPCIBUF[] is available for this function to store intermediate values
# as they are received from the caller, since some checks may require
# values that are not available yet. Managing _RPCIBUF[] is completely up
# to this function, as well as cleaning it up, drop it completeley, or
# anything else. The function will usually store intermediate values as
# _RPCIBUF[name] = value.
#
# It is up to this function to detect the special name "_ENOMSG", that
# is passed to it when no more input remains to be processed.
# =====================================================================

function _invar(name,value,	prefix) {

   if (name == "_ENOMSG") {

      # perform closing operations here, if any.

      return 0
   }

   return(_wwwarg(prefix name,value))
}

# =====================================================================
# string _outvar(string name, string value)
#
# Default output variable post-processing hook. Simply returns the
# passed argument value unchanged.
# When no more arguments are available, the function is called with the
# special argument "_ENOMSG", and the associated value (see csalib.awk).
# The special array _RPCOBUF[] is available for this function to store
# intermediate values as they are received from the caller, since
# some checks may require values that are not available yet. Managing
# _RPCOBUF[] is completely up to this function, as well as cleaning it
# up, drop it completeley, or anything else. The function will usually
# store intermediate values as _RPCOBUF[name] = value.
# =====================================================================

function _outvar(name,value) {

   return value
}

# =====================================================================
# End of rpciolib.awk
# =====================================================================
