# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003,2004,2005 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _formselect.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# string _formselect(array value, array label, string name,\
#	     string selected, int size, boolean multiple, string empty)
#
# Returns the proper HTML "select" form fragment, based on the specified
# arguments. If value[] and label[] contain entries that need to be
# XML-encoded, that must be done by the caller. Position "0" of either
# arrays must contain the No. of select options. If argument "empty"
# is passed, then the a trailing empty <option> element is appended to
# the resulting HTML object, with the specified label and a _NULL value.
#
# Example:
#
# value[0] = 3
# value[1] = "1st cartoon"
# value[2] = "2nd cartoon"
# value[3] = "3rd cartoon"
#
# label[1] = "Donald Duck"
# label[2] = "Mickey Mouse"
# label[3] = "Uncle Scrooge"
#
# print _formselect(value,label,"myselect",value[2])
#
#
# Result:
#
# <select name="myselect">
#  <option value="1st cartoon">Donald Duck</option>
#  <option selected value="2nd cartoon">Mickey Mouse</option>
#  <option value="3rd cartoon">Uncle Scrooge</option>
# </select>
# =====================================================================

function _formselect(a1,a2,name,value,size,multi,empty,		\
					max,count,fragment) {

  count = 0

  # The No. of select entries is taken from element "0" of
  # either arrays.
  max = a1[0]
  if (!max) max = a2[0]

  max += 0; size += 0; multi += 0		# Cast to numbers.

  if (empty != "") {
     a1[++max] = ""
     a2[max] = empty
  }

  if (!max) return _NULL			# Nothing to do.

  if (name == _NULL) name = "select1"		# Default name.

  fragment = "<select name=\"" name "\""

  if (size > 1) fragment = fragment " size=\"" size "\""
  if (multi) fragment = fragment " multiple"

  fragment = fragment ">\n"

  while (++count <= max) {
     fragment = fragment "<option"
     if (value == a1[count]) fragment = fragment " selected"

     fragment = fragment " value=\"" a1[count] "\">" \
	(a2[count] == _NULL ? a1[count] : a2[count]) "</option>\n"
  }

  fragment = fragment "</select>"

  return fragment
}

