/*
   rxpyx.c: XML to PYX validating translator, based on RXP.

   Copyright (C) 2003 Carlo Strozzi <carlos@linux.it>

   Most of this code comes from xmlv.c, written on top of RXP by
   Sean McGrath <sean.mcgrath@propylon.com>.Original code included
   with permission of the author.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   Usage:  rxpyx [-o outfile] [infile|<stdin>]
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>

#ifndef CHAR_SIZE
#define CHAR_SIZE 8
#endif

#include "system.h"
#include "ctype16.h"
#include "charset.h"
#include "string16.h"
#include "dtd.h"
#include "input.h"
#include "xmlparser.h"
#include "stdio16.h"
#include "version.h"
#include "namespaces.h"

#define MARK_I_WHITESPACES 0x0001

/* Compare attribute values (used for sorting attributes for ESIS output)   */
int attr_compare(const void *a, const void *b);
/* Print an event*/
void print_bit(Parser p, XBit bit);

void print_attrs(ElementDefinition e, Attribute a);

/* Handler for newly opened entities*/
InputSource entity_open(Entity ent, void *arg);

Parser p;
Entity ent=0;
XBit bit;
InputSource source = 0;
int flags = 0, retcode = 0, ofd, ifd;		/* Carlo */


/* Output newline and tab characters as escapes*/
/* Required both for attribute values and character data (#PCDATA)*/

static void SanitizeData (const char *s) {
  while (*s) {
    switch (*s) {
    case 10:
      Printf("\\n");
      break;
    case 13:
      break;
    case 9:
      Printf ("\\t");
      break;
    case '\\':
      Printf ("\\\\");
      break;
    default:
      Printf ("%c",*s);
    }
    s++;
#if (CHAR_SIZE == 16)
    s++;
#endif

  }
}

static void usage () {
  fprintf (stderr,"XMLV - XML parsing utility.\n");
  fprintf (stderr,"XML Parser by Robin Tobin\n");
  fprintf (stderr,"ESIS Generation by Sean Mc Grath http://www.digitome.com/sean.html\n");
  fprintf (stderr,"Usage: xmlv [infile] [-o outfile]\n");
}

void ProcessSource (InputSource source) {
  if(ParserPush(p, source) == -1) {
    ParserPerror(p, &p->xbit);
    return;
  }

  while(1) {
    bit = ReadXBit(p);
    /*printf("I am about to call print_bit\n");*/
    print_bit(p, bit);
    if (bit->type == XBIT_eof)
      break;
    if (bit->type == XBIT_error)
      return;
  }
  FreeXBit(bit);
}

int main(int argc, char **argv)
{

  /* FILE *fp; */		/* No longer used. Carlo */

  if ((argc > 1) &&
      (
       strcmp (argv[1],"?")==0 ||
       strcmp (argv[1],"-?")==0 ||
       strcmp (argv[1],"/help")==0 ||
       strcmp (argv[1],"-help")==0 ||
       strcmp (argv[1],"/h")==0 ||
       strcmp (argv[1],"-h")==0 ||
       strcmp (argv[1],"/?")==0)
      ) {
    usage();
    exit(1);
  }

  /*
  if ((argc > 1) && (!(strcmp(argv[1],"-i")))){
      flags = flags | MARK_I_WHITESPACES;
      argv++;
      argc--;
  }
  */

  /* Connect stdout to the specified input file, if requested. Carlo */
  if ((argc > 2) && \
      (!(strcmp(argv[1],"-o")))){
      if ((ofd = creat(argv[2], 0600)) == -1) exit(3);
      if (dup2(ofd,1) == -1) exit(4);
      argv += 2;
      argc -= 2;
  }

  if (argc == 1) {
      p = NewParser();
      ParserSetEntityOpener(p, entity_open);

      ParserSetFlag(p, Validate, 1);
      ParserSetFlag(p, ReturnDefaultedAttributes, 1);
      ParserSetFlag(p, MergePCData, 1);
      ParserSetFlag(p, ErrorOnBadCharacterEntities, 1);    
      ParserSetFlag(p, ErrorOnUndefinedEntities, 1);
      ParserSetFlag(p, XMLStrictWFErrors, 1);
      ParserSetFlag(p, WarnOnRedefinitions, 0);

      /* added by Carlo */
      ParserSetFlag(p, ErrorOnValidityErrors, 1);

      SetFileEncoding(Stdout, CE_UTF_8);
      /* Uncomment the following printf if you would like to
      // see a message on standard error when input is being
      // read from standad input rather than from a file 
      //fprintf (stderr,"Reading from standard input\n");*/
      source = SourceFromStream("<stdin>", stdin);
      ProcessSource (source);
  }
  else {
    for (++argv; argc>1; argc--,argv++) {
        
          p = NewParser();
          ParserSetEntityOpener(p, entity_open);

          ParserSetFlag(p, Validate, 1);
          ParserSetFlag(p, ReturnDefaultedAttributes, 1);
          ParserSetFlag(p, MergePCData, 1);
          ParserSetFlag(p, ErrorOnBadCharacterEntities, 1);    
          ParserSetFlag(p, ErrorOnUndefinedEntities, 1);
          ParserSetFlag(p, XMLStrictWFErrors, 1);
          ParserSetFlag(p, WarnOnRedefinitions, 0);
          ParserSetFlag(p, XMLSpace, 1);

	  /* added by Carlo */
	  ParserSetFlag(p, ErrorOnValidityErrors, 1);

          SetFileEncoding(Stdout, CE_UTF_8);
      /* Uncomment the following fprintf if you want to see
      // filenames appearing on standard error.
      //fprintf (stderr,"File:'%s'\n",*argv);*/

	/* I prefer to always read from <stdin>, even when the
	   input file is passed as a command-line argument,
	   so that eny error msgs from the parser always refer
	   to <stdin>, without exposing real path names to the
	   client. Carlo. */

	if ((ifd = open(*argv, O_RDONLY)) == -1) exit(5);
	if (dup2(ifd,0) == -1) exit(6);

      /*fp = fopen(*argv,"r");
      //if(fp == 0){
      //
      //    fprintf(stderr,"Having trouble accessing \"%s\", Reading from stdin.\n",*argv); */
            source = SourceFromStream("<stdin>", stdin);
            ProcessSource (source);

            FreeDtd(p->dtd);
            FreeParser(p);

            continue;
      /*}
      //    fclose(fp); */
      #ifdef WIN32
        {
            char path[255];
            char *ip;
            char *op;

            ip=*argv;
            op=path;
            if((*argv)[1] == ':') {
                path[0]='/';
                op++;
            }
            while(*ip) {
                if(*ip == '\\')
                    *op='/';
                else
                    *op=*ip;
            ip++;
            op++;
        }
        *op=0;
        ent = NewExternalEntity(0, 0, path, 0, 0);
      }
      #else

        ent = NewExternalEntity(0, 0, *argv, 0, 0);
      #endif
      if(ent) {
        source = EntityOpen(ent);
        ProcessSource (source);
      }
    FreeDtd(p->dtd);
    FreeParser(p);
    }
  }
  return retcode;
}

void print_bit(Parser p, XBit bit)
{
  switch(bit->type) {
  case XBIT_eof:
      break;
  case XBIT_error:
    /*printf("I am about to call ParserPerror!\n");*/
    ParserPerror(p, bit);
    retcode = 1;		/* Carlo */
    break;
  case XBIT_start:
  case XBIT_empty:
    
    Printf("(%S\n", bit->element_definition->name);
    print_attrs(bit->element_definition, bit->attributes);
    if(bit->type == XBIT_empty)
      Printf(")%S\n",bit->element_definition->name);
    break;
  case XBIT_end:
    Printf(")%S\n", bit->element_definition->name);
    break;
  case XBIT_pi:
    Printf("?%S ",bit->pi_name);
    SanitizeData ((char *)bit->pi_chars);
    Printf("\n");
    break;
  case XBIT_pcdata:
    if((bit->i1) && (flags & MARK_I_WHITESPACES)){
        printf("I");
    }
    else{
        printf("-");
    }
    SanitizeData((char *)bit->pcdata_chars);
    printf("\n");
    break;
  case XBIT_cdsect:
    printf("-");
    SanitizeData((char *)bit->cdsect_chars);
    printf("\n");
    break;
  case XBIT_dtd:
  case XBIT_comment:
    break;
  default:
    fprintf(stderr, "\nUnknown event type %s\n", XBitTypeName[bit->type]);
    exit(1);
    break;
  }
  /*printf("Finished in print_bit\n");*/
}

int attr_compare(const void *a, const void *b)
{
    return Strcmp((*(Attribute *)a)->definition->name,
          (*(Attribute *)b)->definition->name);
}

void print_attrs(ElementDefinition e, Attribute a)
{
    Attribute b;
    Attribute *aa;
    int i, n = 0;
    
    /*get the number of attributes*/
    for(b=a; b; b=b->next){
        n++;
    }

    if(n == 0)
        return;

    
    aa = malloc(n * sizeof(*aa));

    for(i=0, b=a; b; i++, b=b->next){
        aa[i] = b;
    }

    qsort((void *)aa, n, sizeof(*aa), attr_compare);

    for(i=0; i<n; i++)
    {
      Printf("A%S ",aa[i]->definition->name);
      SanitizeData ((char *)aa[i]->value);
      Printf("\n");
    }
    free(aa);
}


InputSource entity_open(Entity ent, void *arg)
{
    return EntityOpen(ent);
}

/* EOF */
