/* unescape: copy stdin to stdout, applying the requested unescaping
   of special characters along the way. Of course this could be
   done just as well with a sed(1) script, but the latter would be
   much slower on large input, and could not cope easily with the
   transformation of newlines.

   Copyright (c) 2003 Carlo Strozzi

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   Usage:  unescape [-z]

   where:

     '-z' stands for 'binary zeroes'

   Notes:
   Currently the only unescaping supported is '-z'.

 ****************************************************************************/

static char Psz_RCS[] = "$Date: 2003/06/06 10:40:56 $";

#include <stdio.h>
#include <stdlib.h>

static void usage() {
  fprintf(stderr,"usage: unescape [-z]\n");
}

int main(int argc, char **argv) {

   int c, enc='p', escape=0;

   if ((argc > 1) &&
       (!strcmp(argv[1],"-h") || !strcmp(argv[1],"--help"))) {
      usage();
      exit(1);
   }

   if (argc > 1) {
      if (!strcmp(argv[1],"-z")) enc='z';
      else {
      	 usage();
      	 exit(1);
      }
   }

   if (enc == 'z') {

      /* mawk(1) un-escaping: unescape binary zeroes */

      while ((c = getchar()) != EOF) {

	    if (!escape && c == '\\') {
	       escape = 1;
	       continue;
	    }

	    if (escape && c == '0') c = '\0';

	    putchar(c);
	    escape = 0;
      }
   }

   exit(0);
}

/* End of program */
